%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%    TESTING APPROXIMATE SOLUTIONS TO THE CME        %%%%%%%%%%%%
%%%%%%%%%%%             COMPUTE POWER CURVES                   %%%%%%%%%%%%
%%%%%%%%%%%          Authors: Jenkinson/Goutsias               %%%%%%%%%%%%
%%%%%%%%%%%           Last Modified: 02/25/2013                %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function beta = findPower(mu1,sigma1,mu2,sigma2,alpha,L,numMCsamples,rVals)

count = 1;

beta  = zeros(size(rVals));

for r = rVals
    Htot = 0;
    for iter = 1:numMCsamples
        %
        % Generate L samples from Q_r
        %
        YY = zeros(L,1);
        for l=1:L
            if rand<r
                YY(l) = sigma1*randn+mu1;
            else
                YY(l) = sigma2*randn+mu2;
            end
        end

        %
        % Normalize samples to standard normal
        %
        ZZ = (YY-mu1)/sigma1;
        
        %
        % Perform test
        %
        
        H    = kstest(ZZ,[],alpha);  
        Htot = Htot + H;
    end
    
    beta(count) = Htot/numMCsamples; % fraction of times null rejected
    count = count+1;
end