%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%    TESTING APPROXIMATE SOLUTIONS TO THE CME        %%%%%%%%%%%%
%%%%%%%%%%%         SOLVE SCHLOGL MODEL USING KSA              %%%%%%%%%%%%
%%%%%%%%%%%          Authors: Jenkinson/Goutsias               %%%%%%%%%%%%
%%%%%%%%%%%           Last Modified: 02/24/2013                %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function [Q,pNext] = Schlogl(tf,timeStep,gamma2,gamma3,V,k1,k2,k3,k4,x0)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize simulation parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

N         = 1500;  % Non-absorbing state of Finite State Projection
numStates = N+1;   % number of states with nonzero probability in state space
tol       = 1e-30; % EXPOKIT tol parameter
K0        = 30;    % Krylov subspace dimension

display('Running Expokit. Please wait...');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make generator matrix of Markov process
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

row = zeros(3*numStates,1);
col = zeros(3*numStates,1);
value = zeros(3*numStates,1);
count = 1;

for XplusOne = 1:N % make non-absorbing states of the FSP
    
        %
        % find propensities
        %
        X = (XplusOne-1);
        
        propensity(1)=k1*gamma2*X*(X-1)/V;
        propensity(2)=k2*X*(X-1)*(X-2)/(V^2);
        propensity(3)=k3*X;
        propensity(4)=k4*V*gamma3;
            
        %
        % add value to diagonal
        %
        diagVal      = XplusOne;
        row(count)   = diagVal;
        col(count)   = diagVal;
        value(count) = -1*sum(propensity);
        count        = count+1;

        %
        % add propensity for reaction 1 and reaction 3
        %

        if propensity(1)>0
            % the location with one more X molecule
            location1    = XplusOne+1;            
            row(count)   = location1;
            col(count)   = diagVal;
            value(count) = propensity(1)+propensity(4);
            count        = count+1;
        end

        %
        % add propensity for reaction 2 and reaction 4
        %

        if propensity(2)>0
            % the location with one less X molecule
            location2    = XplusOne-1;            
            row(count)   = location2;
            col(count)   = diagVal;
            value(count) = propensity(2)+propensity(3);
            count        = count+1;
        end
        
end

Q = sparse(row(value~=0),col(value~=0),value(value~=0),numStates,numStates);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set initial condition to have X(0)=x0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

p0           = zeros(size(Q,1),1);
XplusOne     = x0+1;
p0(XplusOne) = 1;
pPrev        = p0;
timeCount    = 1;  

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% USE EXPOKIT software package
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

for t= timeStep:timeStep:tf

    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Use EXPOKIT to take timestep
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    [pNext,~,~] = mexpv(timeStep,Q,pPrev,tol,K0);
    pPrev = pNext;
    timeCount = timeCount+1;
    
end