%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%    TESTING APPROXIMATE SOLUTIONS TO THE CME        %%%%%%%%%%%%
%%%%%%%%%%%   DRAW MONTE CARLO SAMPLES FROM SCHLOGL MODEL      %%%%%%%%%%%%
%%%%%%%%%%%          Authors: Jenkinson/Goutsias               %%%%%%%%%%%%
%%%%%%%%%%%           Last Modified: 02/24/2013                %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function XX = monteCarlo(gamma2,gamma3,V,k1,k2,k3,k4,x0,tf,L)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

t         = 0;           % time variable
X         = x0;          % X(t)
XX        = zeros(L,1);  % Monte Carlo Samples

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Monte Carlo iterations 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

for l=1:L
    while t < tf
        
        %
        % propensity functions
        %
        propensity(1) = k1*gamma2*X*(X-1)/V;
        propensity(2) = k2*X*(X-1)*(X-2)/(V^2);
        propensity(3) = k3*X;
        propensity(4) = k4*V*gamma3;
        
        %
        % Update state
        %
        astr  = sum(propensity);
        if ~astr % Simulation over
            t = tf;
        else
            
            tau = exprnd(inv(astr));
            t   = t + tau;
            m   = find(cumsum(propensity) > astr*rand, 1 );
            
            if m==1 || m==4
                X = X + 1; 
            else
                X = X - 1; 
            end
        end
        
    end 
        
    XX(l) = X;
    
    t=0;  % reset time to begining for next iteration
    X=x0; % re-initailize X for next iteration
end