%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%    TESTING APPROXIMATE SOLUTIONS TO THE CME        %%%%%%%%%%%%
%%%%%%%%%%%    DRAW MONTE CARLO SAMPLES FROM SIR MODEL         %%%%%%%%%%%%
%%%%%%%%%%%          Authors: Jenkinson/Goutsias               %%%%%%%%%%%%
%%%%%%%%%%%           Last Modified: 02/27/2013                %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function XX = monteCarlo(k1,k2,Omega,x0,tf,L)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

t  = 0;               % time variable
X  = x0;              % Species numbers
XX = zeros(L,2);      % Monte Carlo Samples

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Monte Carlo iterations 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

for l=1:L
    while t < tf

        %
        % compute propensity functions
        %

        propensity(1)= k1*X(1)*X(2)/Omega;
        propensity(2)= k2*X(2);
        
        %
        % Update state
        %
        %
        % Update state
        %
        
        astr  = sum(propensity);
        if ~astr % simulation over

            t = tf;

        else % take next Monte Carlo step

            tau = exprnd(inv(astr));
            t   = t + tau;
            m   = find(cumsum(propensity) > astr*rand, 1 );

            %
            %  Update state
            %
            
            if m==1
                X(1) = X(1)-1;
                X(2) = X(2)+1;
            else
                X(2) = X(2) - 1;
            end
            
        end

    end 
        
    XX(l,:) = X; %store result
    
    t = 0;     % reset t to begining for next iteration
    X = x0;    % reset Y to begining for next iteration
end